@@ -53,7 +53,7 @@
 #'   Symbol \eqn{\phi} denotes an AR coefficient. Note that regardless of any constraints, the nominal order of AR coefficients is alway \code{p} for all regimes.
 #'   This argument is ignored if \code{constraints==FALSE}.
 #' @param conditional an (optional) logical argument specifying wether the conditional or exact log-likehood function should be used. Default is \code{TRUE}.
-#' @param boundaries an (optional) logical argument. If \code{TRUE} then \code{loglikelihood} returns -99999 if...
+#' @param boundaries an (optional) logical argument. If \code{TRUE} then \code{loglikelihood} returns \code{minval} if...
 #' \itemize{
 #'   \item any component variance is not larger than zero,
 #'   \item any parametrized mixing weight \eqn{\alpha_{1},...,\alpha_{M-1}} is not larger than zero,
@@ -69,6 +69,8 @@
 #' @param epsilon an (optional) negative real number specifying the logarithm of the smallest positive non-zero number that will be
 #'  handled without external packages. Too small value may lead to a failure or biased results and too large value will make the code
 #'  run significantly slower. Default is \code{round(log(.Machine$double.xmin)+10)} and should not be adjusted too much.
+#' @param minval a negative real number defining the log-likelihood value that will be returned with \code{boundaries==TRUE}
+#'  when the parameter is out of the boundaries. Ignored if \code{boundaries==FALSE}.
 #' @return
 #'  \describe{
 #'   \item{By default:}{log-likelihood value of the specified GMAR or StMAR model,}
@@ -85,7 +87,7 @@
 #'    \item References regarding the StMAR model and general linear constraints will be updated after they are published.
 #'  }
 
-loglikelihood_int <- function(data, p, M, params, StMAR=FALSE, restricted=FALSE, constraints=FALSE, R, conditional=TRUE, boundaries=FALSE, checks=TRUE, returnTerms=FALSE, epsilon) {
+loglikelihood_int <- function(data, p, M, params, StMAR=FALSE, restricted=FALSE, constraints=FALSE, R, conditional=TRUE, boundaries=FALSE, checks=TRUE, returnTerms=FALSE, epsilon, minval) {
 
   if(missing(epsilon)) {
     epsilon = round(log(.Machine$double.xmin)+10)
@@ -128,25 +130,24 @@ loglikelihood_int <- function(data, p, M, params, StMAR=FALSE, restricted=FALSE,
   rownames(pars) = NULL
   sigmas = pars[p+2,]
 
-  # Return -99999 if parameters are out of their boundaries.
+  # Return minval if parameters are out of their boundaries.
   if(boundaries==TRUE) {
     if(any(pars[p+2,]<=0)) {
-      return(-99999)
+      return(minval)
     } else if(M>=2 & sum(alphas[-M])>=1) {
-      return(-99999)
+      return(minval)
     } else if(any(alphas<=0)) {
-      return(-99999)
+      return(minval)
     } else if(!isStationary_int(p, M, params)) {
-      return(-99999)
+      return(minval)
     }
     if(StMAR==TRUE) {
       if(any(dfs<=2)) {
-        return(-99999)
+        return(minval)
       }
     }
   }
 
-  # Check these only if checks == TRUE
   if(checks==TRUE) {
     if(constraints==TRUE) {
       checkConstraintMat(p=p, M=M, R=R, restricted=restricted)
